<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\admin\UserModel;
use App\admin\FollowerModel;
use DataTables;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Storage;
use Illuminate\Validation\Rule;


class UserController extends Controller
{

    //=========================== Display Users data =====================================
    public function users(Request $request)
    {
        if ($request->ajax()) {
            $data = UserModel::where("is_delete", 0)->orderBy('insert_date', 'desc')->get();
            $timezonevalue = config('constants.ADMIN_DATETIMEFORMAT');

            // foreach ($data as $row_data) {
            //     $row_data->updated_datetime = $this->UserModel->date_convert($row_data->updated_datetime, ADMIN_DATETIMEFORMAT, $this->session->userdata(ADMIN_TIMEZONE));
            // }
            return Datatables::of($data)
                ->addIndexColumn()
                ->addColumn('action', function ($row) {
                    $viewUrl = url('admin/user-view/' . $row->id);
                    $deleteUrl = url('admin/user-delete/' . $row->id);
                    $editUrl = url("admin/user-edit/" . $row->id);
                    $btn = '<a href="' . $viewUrl . '" class="view btn btn_color btn-sm"><i class="md md-remove-red-eye"></i></a> ';
                    $btn .= '<a href="' . $editUrl . '" class="edit btn btn_color btn-sm"><i class="md md-mode-edit"></i></a> ';
                    $btn .= '<a href="' . $deleteUrl . '" class="edit btn btn_color btn-sm del_btn" id=' . $row->id . '><i class="md md-delete"></i></a>';
                    return $btn;
                })
                ->rawColumns(['action'])
                ->make(true);
        }

        return view('admin.pages.user');
    }
    /**Date convert on specific timezone.*/
    function date_convert($date, $dateformat, $timezone = NULL)
    {
        $timezone = !empty($timezone) ? $timezone : 'UTC';
        if ($date == '0000-00-00 00:00:00') {
            return $date;
        }
        $date = new DateTime($date);
        $date->setTimezone(new DateTimeZone($timezone));
        return $date->format($dateformat);
    }

    //================================= Add User ==============================
    public function addUsers()
    {
        return view('admin.pages.useradd');
    }

    public function addUsersData(Request $req)
    {
        //dd($req->input('fullname'));
        $req->validate(
            [
                'fullname' => 'required|min:3|max:128',
                'email' => 'required|email|unique:tbl_user',
                'country_code' => 'required|numeric',
                'phone_number' => 'required|numeric',
                'password' => 'required',
                'type' => 'required',
            ],
            [
                'fullname.required' => 'Enter Fullname',
                'email.required' => 'Enter Email ID',
                'email.email' => 'The email must be a valid Email ID',
                'country_code.required' => 'Enter Country Code',
                'phone_number.required' => 'Enter Phone Number',
                'password.required' => 'Enter Password',
                'type.required' => 'Select User Type',
                'email.unique' => 'Email id already exits',
            ]
        );

        $fullname = $req->input('fullname');
        $email = $req->input('email');
        $countey_code = $req->input('country_code');
        $phone_number = $req->input('phone_number');
        $password = $req->input('password');
        $type = $req->input('type');
        // dd($type);
        UserModel::create([
            'full_name' => $fullname,
            'email' => $email,
            'country_code' => $countey_code,
            'mobile_number' => $phone_number,
            'password' => Hash::make($password),
            'user_type' => $type,
            'status_id' => 0,
            'chat_status' => 0,
        ]);
        return response()->json(['status' => 'success', 'message' => 'Data Update Successfully']);
    }
    //======================== View User ==================================
    public function viewUsers($id)
    {
        //dd($id);
        $data = UserModel::where('is_delete', 0)->where('id', $id)->first();
        //dd(config('image.url.profile') . $data->profile_image);

        $follower = FollowerModel::where('follow_id', $id)->count();
        $following = FollowerModel::where('user_id', $id)->count();
        //dd($follower, $following);
        return view('admin.pages.userview', compact('data', 'follower', 'following'));

    }

    //======================== Edit User =================================
    public function editUser($id)
    {
        $data = UserModel::where('is_delete', 0)->where('id', $id)->first();

        return view('admin.pages.useredit', compact('data'));
    }

    public function updateUser(Request $req)
    {
        // dd($req->input('user_id'));
        $req->validate(
            [
                'fullname' => 'required|min:3|max:128',
                'email' => 'required|email',
                'country_code' => 'required|numeric',
                'phone_number' => 'required|numeric',
                'type' => 'required',
            ],
            [
                'fullname.required' => 'Enter Fullname',
                'email.required' => 'Enter Email ID',
                'email.email' => 'The email must be a valid Email ID',
                'country_code.required' => 'Enter Country Code',
                'phone_number.required' => 'Enter Phone Number',
                'type.required' => 'Select Type',
            ]
        );

        $user = UserModel::find($req->input('user_id'));

        $user->full_name = $req['fullname'];
        $user->email = $req['email'];
        $user->country_code = $req['country_code'];
        $user->mobile_number = $req['phone_number'];
        $user->user_type = $req['type'];

        $user->save();
        return response()->json(['status' => 'success', 'message' => 'Data Update Successfully']);
    }

    //============================ Approve User ====================================
    public function approveUser(Request $request)
    {
        $user = UserModel::find($request->input('id'));

        if ($user) {
            $user->is_approve = $request->input('value');
            $user->save();

            return response()->json(['status' => 'success']);
        } else {
            return response()->json(['status' => 'error'], 404);
        }
    }

    //============================ Chnage Status ====================================
    public function statusUser(request $req)
    {
        $data = UserModel::find($req->input('id'));
        if ($data) {
            $data->is_active = ($req->input('status') == '0') ? '1' : '0';
            $data->save();

            $data = UserModel::where('is_delete', 0)->where('id', $req->input('id'))->get();

            return response()->json(['status' => 'success', 'data' => $data]);
        } else {
            return response()->json(['status' => 'error'], 404);
        }
    }

    //================================ Delete User ==================================
    public function deleteUser(request $req)
    {
        //dd($req->input('id'));
        $user = UserModel::find($req->input('id'));

        if ($user) {
            $user->is_delete = '1';
            $user->save();
            return response()->json(['status' => 'success']);
        }

        return response()->json(['status' => 'error', 'message' => 'Food not found.'], 404);
    }

}
